import sys
from libcloud.compute.types import Provider
from libcloud.compute.providers import get_driver
from libcloud.dns.types import Provider as DnsProvider
from libcloud.dns.providers import get_driver as get_dns_driver
from prettytable import PrettyTable
from vmanage.tencent import TencentDriver
import json
import os
import csv
import time
import datetime

NodesTableHeaders = ['Name', 'Ip', 'State', 'Size', 'Region', 'Created On']
RecordsTableHeaders = ['Zone', 'Record Name', 'Record Type']
AWSZones = ["us-east-1", "us-east-2", "us-west-1", "us-west-2", "ap-south-1", "ap-northeast-2", "ap-southeast-1",
            "ap-southeast-2", "ap-northeast-1", "ca-central-1", "eu-central-1", "eu-west-1", "eu-west-2", "eu-west-3",
            "sa-east-1"]
AliRegions = ["cn-hangzhou", "cn-shanghai", "cn-qingdao", "cn-beijing", "cn-shenzhen", "cn-hongkong", "ap-southeast-1", "ap-southeast-2", "ap-southeast-3",
              "ap-southeast-5", "ap-northeast-1", "ap-south-1", "me-east-1"]
Users = ["albert", "erez", "maurice", "david", "biderman", "idan", "orr", "oren", "shay", "assaf",
        "moriya", "omer", "moran", "moshe", "tc-qa", "karin", "ashimano"]


def get_pretty_table(table_headers):
    t = PrettyTable(table_headers)
    t.align = 'l'
    return t

def set_nodes_driver(driver):
    nodes = driver.list_nodes()
    for node in nodes:
        node.driver = driver
    return nodes

def get_cloud_nodes(account, provider, unnamed):
    nodes = list()
    res_nodes = list()

    if provider == "amazon":
        cls = get_driver(Provider.EC2)
        for zone in AWSZones:
            driver = cls(account[provider]['apiKey'], account[provider]['apiSecret'], region=zone)
            nodes = nodes + set_nodes_driver(driver)
    elif provider == "aliyun":
        cls = get_driver(Provider.ALIYUN_ECS)
        for region in AliRegions:
            driver = cls(account[provider]['apiKey'], account[provider]['apiSecret'], region=region)
            nodes = nodes + set_nodes_driver(driver)
    elif provider == "tencent":
        driver = TencentDriver(account[provider]['SecretId'], account[provider]['SecretKey'], "")
        regions = driver.list_regions()
        for region in regions:
            driver = TencentDriver(account[provider]['SecretId'], account[provider]['SecretKey'], region.Region)
            nodes = nodes + set_nodes_driver(driver)
        for node in nodes:
            node.name = node.InstanceName
    else:
        if provider == 'google':
            dir_path = os.path.dirname(os.path.realpath(__file__))
            cls = get_driver(Provider.GCE)
            project = account[provider]['project']
            driver = cls(account[provider]['clientEmail'], dir_path + '/' + account[provider]['keyfile'], project, project=project)
        elif provider == "digitalocean":
            cls = get_driver(Provider.DIGITAL_OCEAN)
            driver = cls(account[provider]['apiKey'], api_version='v2')
        elif provider == "vultr":
            cls = get_driver(Provider.VULTR)
            driver = cls(account[provider]['apiKey'])
        elif provider == "softlayer":
            cls = get_driver(Provider.SOFTLAYER)
            driver = cls(account[provider]['username'], account[provider]['apiKey'])
        elif provider == "packet":
            cls = get_driver(Provider.EQUINIXMETAL)
            driver = cls(account[provider]['apiKey'])
        elif provider == "azure":
            cls = get_driver(Provider.AZURE_ARM)
            driver = cls(tenant_id=account[provider]['tenant'], subscription_id=account[provider]['subscription'], 
                   key=account[provider]['apiKey'], secret=account[provider]['apiSecret'])
        nodes = set_nodes_driver(driver)

    if unnamed:
        for node in nodes:
            name_found = False
            for user in Users:
                if user in node.name.lower():
                    name_found = True
                    break
            if not name_found:
                res_nodes.append(node)
        return res_nodes
    else:
        return nodes


def print_digital_ocean_nodes(account, name, unnamed):
    nodes = [node for node in get_cloud_nodes(account, "digitalocean", unnamed) if name is None or name in node.name.lower()]
    print('\nDigitalOcean - ' + account['name'])
    t = get_pretty_table(NodesTableHeaders)
    for node in nodes:
        if len(node.public_ips) == 0 or node.public_ips[0] is None:
            t.add_row([node.name, ' ' , node.state, node.extra['size_slug'], node.extra['region'],
                       node.extra['created_at']])
        else:
            t.add_row([node.name, node.public_ips[0], node.state, node.extra['size_slug'], node.extra['region'],
                       node.extra['created_at']])
    print(t)


def print_google_nodes(account, name, unnamed):
    nodes = [node for node in get_cloud_nodes(account, "google", unnamed) if name is None or name in node.name.lower()]
    print('\nGoogle - ' + account["google"]["project"])
    t = get_pretty_table(NodesTableHeaders)
    for node in nodes:
        if len(node.public_ips) == 0 or node.public_ips[0] is None:
            t.add_row([node.name, ' ', node.state, node.size, node.extra['zone'].name,
                       node.extra['creationTimestamp']])
        else:
            t.add_row([node.name, node.public_ips[0], node.state, node.size, node.extra['zone'].name, 
                       node.extra['creationTimestamp']])
    print(t)


def print_amazon_nodes(account, name, unnamed):
    nodes = [node for node in get_cloud_nodes(account, "amazon", unnamed) if name is None or name in node.name.lower()]
    print('\nAmazon - ' + account["name"])
    t = get_pretty_table(NodesTableHeaders)
    for node in nodes:
        t.add_row([node.name, node.public_ips, node.state, node.extra['instance_type'], node.extra['availability'],
                   node.extra['launch_time']])
    print(t)


def print_aliyun_nodes(account, name, unnamed):
    nodes = [node for node in get_cloud_nodes(account, "aliyun", unnamed) if name is None or name in node.name.lower()]
    print('\nAliyun - ' + account["name"])
    t = get_pretty_table(NodesTableHeaders)
    for node in nodes:
        t.add_row([node.name, node.public_ips, node.state, node.extra["instance_type"], node.extra['zone_id'],
                   node.extra['creation_time']])
    print(t)

def print_tencent_nodes(account, name, unnamed):
    nodes = [node for node in get_cloud_nodes(account, "tencent", unnamed) if name is None or name in node.name.lower()]
    print('\nTencent - ' + account["name"])
    t = get_pretty_table(NodesTableHeaders)
    for node in nodes:
        t.add_row([node.InstanceName, node.PublicIpAddresses, node.InstanceState, node.InstanceType, node.Placement.Zone, node.CreatedTime]) 
    print(t)

def print_azure_nodes(account, name, unnamed):
    nodes = [node for node in get_cloud_nodes(account, "azure", unnamed) if name is None or name in node.name.lower()]
    print('\nAzure - ' + account["name"])
    t = get_pretty_table(NodesTableHeaders)
    for node in nodes:
        t.add_row([node.name, node.public_ips, node.state, node.extra["properties"]["hardwareProfile"]["vmSize"], node.extra["location"], ""])
    print(t)

def print_vultr_nodes(account, name, unnamed):
    nodes = [node for node in get_cloud_nodes(account, "vultr", unnamed) if name is None or name in node.name.lower()]
    print('\nVultr - ' + account["name"])
    t = get_pretty_table(NodesTableHeaders)
    for node in nodes:
        if len(node.public_ips) == 0 or node.public_ips[0] is None:
            t.add_row([node.name, "", node.state, "", node.extra["location"], ""])
        else:
            t.add_row([node.name, node.public_ips[0], node.state, "", node.extra["location"], ""])
    print(t)


def print_softlayer_nodes(account, name, unnamed):
    nodes = [node for node in get_cloud_nodes(account, "softlayer", unnamed) if name is None or name in node.name.lower()]
    print('\nSoftLayer - ' + account["name"])
    t = get_pretty_table(NodesTableHeaders)
    for node in nodes:
        if len(node.public_ips) == 0 or node.public_ips[0] is None:
            t.add_row([node.extra['hostname'], ' ', node.state, '', node.extra['datacenter'],
                       node.extra['created']])
        else:
            t.add_row([node.extra['hostname'], node.public_ips[0], node.state, '', node.extra['datacenter'],
                       node.extra['created']])
    print(t)


def print_packet_nodes(account, name, unnamed):
    nodes = [node for node in get_cloud_nodes(account, "packet", unnamed) if name is None or name in node.name.lower()]
    print('\nPacket - ' + account["name"])
    t = get_pretty_table(NodesTableHeaders)
    for node in nodes:
        if len(node.public_ips) == 0 or node.public_ips[0] is None:
            t.add_row([node.name, "", node.state, node.size.id, node.extra["facility"]["code"], node.extra["created_at"]])
        else:
            t.add_row([node.name, node.public_ips[0], node.state, node.size.id, node.extra["facility"]["code"], node.extra["created_at"]])
    print(t)


def print_ns1_records(account_name, provider, name, apikey):
    if provider is None or provider == 'ns1':
        cls = get_dns_driver(DnsProvider.NSONE)
        driver = cls(apikey)

        print('\nNS1 - ' + account_name)
        t = get_pretty_table(RecordsTableHeaders)
        zones = driver.list_zones()
        for zone in zones:
            records = [record for record in driver.list_records(zone) if name is None or name in record.name.lower()]
            for record in records:
                t.add_row([record.zone.domain, record.name, record.type])
        print(t)


def print_resources(accounts, provider, resource_name, unnamed):
    for account in accounts:
        if provider is None or provider == "digitalocean":
            print_digital_ocean_nodes(account, resource_name, unnamed)
        if provider is None or provider == "vultr":
            print_vultr_nodes(account, resource_name, unnamed)
        if provider is None or provider == "softlayer":
            print_softlayer_nodes(account, resource_name, unnamed)
        if provider is None or provider == "packet":
            print_packet_nodes(account, resource_name, unnamed)
        if provider is None or provider == "google":
            print_google_nodes(account, resource_name, unnamed)
        if provider is None or provider == "amazon":
            print_amazon_nodes(account, resource_name, unnamed)
        if provider is None or provider == "aliyun":
            print_aliyun_nodes(account, resource_name, unnamed)
        if provider is None or provider == "tencent":
            print_tencent_nodes(account, resource_name, unnamed)
        if provider is None or provider == "azure":
            print_azure_nodes(account, resource_name, unnamed)
        if 'ns1' in account and provider == "ns1":
            print_ns1_records(account['name'], provider, resource_name, account['ns1']['apiKey'])


def get_accounts_config(account):
    accounts = list()
    config = None
    dir_path = os.path.dirname(os.path.realpath(__file__))
    # Read JSON configuration file
    with open(dir_path + '/config.json') as json_data:
        config = json.load(json_data)

    # Get relevant account information from the config file
    if account == 'qa' or account is None:
        accounts.append(config['qa'])
    if account == 'tms' or account is None:
        accounts.append(config['tms'])
    if account == 'tcr' or account is None:
        accounts.append(config['tcr'])

    return accounts

def delete_azure_node(node, config):
    node.driver.destroy_node(node)
    
    # Delete public IP's
    public_ips = [public_ip for public_ip in node.driver.ex_list_public_ips(config["resourceGroup"]) if node.name in public_ip.name.lower()]
    for ip in public_ips:
        node.driver.ex_delete_public_ip(ip)
        

    # Delete disks
    disks = [disk for disk in node.driver.list_volumes() if node.name in disk.name.lower()]
    for disk in disks:
        node.driver.destroy_volume(disk)
   
def stop_instances(accounts, providerList, resource_name):
    # Can't really stop anything in NS1
    if 'ns1' in providerList:
        providerList.pop(providerList.index('ns1'))

    for provider in providerList:
        nodes = [node for node in get_cloud_nodes(accounts[0], provider, False) if resource_name is None or resource_name in node.name.lower()]

        if nodes:
            print("Current provider is: " + provider + ". The following " + str(len(nodes)) + " nodes will be stopped:")
            for node in nodes:
                print(node.name)
            answer = input("Stop all nodes? Y/N: ")
            if answer.lower() == "y":
                for node in nodes:
                    print("Stopping node ", node.name)
                    node.driver.stop_node(node)
            else:
                print("Skipped stopping nodes for provider ", provider)


def delete_instances(accounts, providerList, resource_name):
    ns1 = ""
    if 'ns1' in providerList:
        ns1 = providerList.pop(providerList.index('ns1'))

    for provider in providerList:
        nodes = [node for node in get_cloud_nodes(accounts[0], provider, False) if resource_name is None or resource_name in node.name.lower()]
        if nodes:
            print("Current provider is: " + provider + ". The following " + str(len(nodes)) + " nodes will be deleted:")
            for node in nodes:
                print(node.name)
            answer = input("Delete all nodes? Y/N: ")
            if answer.lower() == "y":
                for node in nodes:
                    print("Deleting node ", node.name)
                    if provider == 'azure':
                        delete_azure_node(node, accounts[0]['azure'])
                    else:
                        node.driver.destroy_node(node)
            else:
                print("Skipping nodes deletion for provider ", provider)

    if ns1:
        apikey = accounts[0]['ns1']['apiKey']
        cls = get_dns_driver(DnsProvider.NSONE)
        driver = cls(apikey)
        zones = driver.list_zones()
        for zone in zones:
            records = [record for record in driver.list_records(zone) if resource_name in record.name.lower()]
            if records:
                print("The following DNS records for zone will be deleted:", len(records), zone.domain)
                for record in records:
                    print(record.name)
                answer = input("Delete all records? Y/N: ")
                if answer.lower() == "y":
                    for record in records:
                        print("Deleting DNS record ", record.name)
                        driver.delete_record(record)
                else:
                    print("Skipping deletion for zone ", zone.domain)

    return None


def count_instances_per_account(account):

    # digitalocean
    apikey = account['digitalocean']['apiKey']
    cls = get_driver(Provider.DIGITAL_OCEAN)
    dgDriver = cls(apikey, api_version='v2')
    dgCount = len(dgDriver.list_nodes())
    print("dgCount = ", dgCount)

    # vultr:
    vCount = 0
    if not 'tcr' in account['name']:
        apikey = account['vultr']['apiKey']
        cls = get_driver(Provider.VULTR)
        vDriver = cls(apikey)
        vCount = len(vDriver.list_nodes())
        print("vCount = ", vCount)

    # google:
    dir_path = os.path.dirname(os.path.realpath(__file__))
    cls = get_driver(Provider.GCE)
    client_email, keyfile, project = account['google']['clientEmail'], account['google']['keyfile'], \
                                     account['google']['project']
    gDriver = cls(client_email, dir_path + '/' + keyfile, project, project=project)
    gCount = len(gDriver.list_nodes())
    print("gCount = ", gCount)

    # softlayer:
    username, apikey = account['softlayer']['username'], account['softlayer']['apiKey']
    cls = get_driver(Provider.SOFTLAYER)
    slDriver = cls(username, apikey)
    sCount = len(slDriver.list_nodes())
    print("sCount = ", sCount)

    # amazon:
    cls = get_driver(Provider.EC2)
    apikey, apisecret = account['amazon']['apiKey'], account['amazon']['apiSecret']
    aCount = 0
    for zone in AWSZones:
        aDriver = cls(apikey, apisecret, region=zone)
        aCount += len(aDriver.list_nodes())
    print("aCount = ", aCount)
   
    # aliyun
    cls = get_driver(Provider.ALIYUN_ECS)
    apikey, apisecret = account['aliyun']['apiKey'], account['aliyun']['apiSecret']
    aCount = 0
    for region in AliRegions:
        aDriver = cls(apikey, apisecret, region=region)
        aCount += len(aDriver.list_nodes())
    print("alCount = ", aCount)

    # azure
    cls = get_driver(Provider.AZURE_ARM)
    azDriver = cls(tenant_id=account['azure']['tenant'], subscription_id=account['azure']['subscription'],
                   key=account['azure']['apiKey'], secret=account['azure']['apiSecret']) 
    azCount = len(azDriver.list_nodes())
    print("azCount = ", azCount)
     
    # ns1
    nsCount = 0
    if not 'tcr' in account['name']:
        apikey = account['ns1']['apiKey']
        cls = get_dns_driver(DnsProvider.NSONE)
        driver = cls(apikey)
        zones = driver.list_zones()
        for zone in zones:
            records = driver.list_records(zone)
            nsCount += len(records)
        print("nsCount = ", nsCount)

    fileName = account['name'] + "_instances_count.csv"
    fullPath = "/var/www/" + fileName
    with open(fullPath, "a") as csvFile:
        writer = csv.writer(csvFile)
        writer.writerow([datetime.date.today(),dgCount,vCount,gCount,sCount,aCount,nsCount])


def count_instances(accounts):
    for account in accounts:
        count_instances_per_account(account)
